/* Automatically generated by refactor.pl.
 *
 *
 * CMDNAME: maillock
 * CMDLEVEL: QCMD_OPER
 * CMDARGS: 3
 * CMDDESC: Set/unset/list mail lock patterns
 * CMDFUNC: csu_domaillock
 * CMDPROTO: int csu_domaillock(void *source, int cargc, char **cargv);
 * CMDHELP: Usage: maillock <-list|-add|-del> <pattern> [<reason>]
 * CMDHELP: Manipulates the list of mail lock patterns.
 * CMDHELP: Any email address matching the pattern will be refused in HELLO/EMAIL.
 * CMDHELP: A reason is optional for -list, but recommended.
 */

#include "../chanserv.h"
#include "../../lib/irc_string.h"
#include <stdio.h>
#include <string.h>

int csu_domaillock(void *source, int cargc, char **cargv) {
  nick *sender=source;
  char *pattern;
  maillock *mlp;

  if (cargc<2) {
    chanservstdmessage(sender,QM_NOTENOUGHPARAMS,"maillock");
    return CMD_ERROR;
  }

  pattern = cargv[1];

  if(!strcasecmp(cargv[0], "-list")) {
    char timebuf[TIMELEN];
    reguser *rup;

    chanservstdmessage(sender, QM_MAILLOCKHEADER);
    for(mlp=maillocks;mlp;mlp=mlp->next) {
      if(match(pattern, mlp->pattern->content))
        continue;

      rup=findreguserbyID(mlp->createdby);
      q9strftime(timebuf,sizeof(timebuf),mlp->created);

      chanservsendmessage(sender, "%-50s %-15s %-25s %s", mlp->pattern->content, rup?rup->username:"??", mlp->reason?mlp->reason->content:"(none)", timebuf);
    }
  } else if(!strcasecmp(cargv[0], "-del")) {
    maillock *pmlp = NULL;

    for(mlp=maillocks;mlp;pmlp=mlp,mlp=mlp->next)
      if(!strcasecmp(pattern, mlp->pattern->content))
        break;

    if(!mlp) {
      chanservstdmessage(sender, QM_MAILLOCKDOESNTEXIST);
      return CMD_ERROR;
    }

    csdb_deletemaillock(mlp);
    if(!pmlp) {
      maillocks=mlp->next;
    } else {
      pmlp->next=mlp->next;
    }
    freemaillock(mlp);

  } else if(!strcasecmp(cargv[0], "-add")) {
    char *reason;
    reguser *rup = getreguserfromnick(sender);;

    if(cargc<3) {
      reason = NULL;
    } else {
      reason = cargv[2];
    }

    for(mlp=maillocks;mlp;mlp=mlp->next) {
      if(!strcasecmp(pattern, mlp->pattern->content)) {
        chanservstdmessage(sender, QM_MAILLOCKALREADYEXISTS);
        return CMD_ERROR;
      }
    }

    mlp=getmaillock();
    mlp->pattern=getsstring(pattern, 300);
    if(reason) {
      mlp->reason=getsstring(reason, 300);
    } else {
      mlp->reason=NULL;
    }
    mlp->createdby=rup->ID;
    mlp->created=time(NULL);
    mlp->id=++lastmaillockID;
    mlp->next=maillocks;
    maillocks=mlp;
    csdb_createmaillock(mlp);

  } else {
    chanservsendmessage(sender, "Invalid option, see HELP.");
    return CMD_ERROR;
  }

  chanservstdmessage(sender, QM_DONE);
  return CMD_OK;
}
